@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 import os
 import pathlib
+from typing import Optional, List
 
 import zgitignore
 
@@ -11,8 +12,6 @@ from ddb.context import context
 from ddb.event import events
 from ddb.utils.file import force_remove
 
-repository = "inetum-orleans/docker-devbox"
-
 
 class UpdateGitignoreAction(InitializableAction):
     """
@@ -37,106 +36,30 @@ class UpdateGitignoreAction(InitializableAction):
         if caches.has("gitignore"):
             caches.unregister("gitignore", callback=lambda c: c.close())
 
-    @staticmethod
-    def find_gitignores(target: str):
-        """
-        Find writable .gitignore files, walking directories up to project home.
-        """
-        dirname = target
-
-        while os.path.abspath(dirname).startswith(config.paths.project_home) and \
-                os.path.abspath(dirname) != os.path.abspath(config.paths.project_home):
-            dirname = os.path.dirname(dirname)
-            gitignore = os.path.join(dirname, ".gitignore")
-            if os.path.exists(gitignore) and os.access(gitignore, os.W_OK):
-                yield gitignore
-
-    @staticmethod
-    def is_file_ignored(file: str, gitignores=None):
-        """Check if a file is ignored, going throw all provided gitignore files"""
-        if not gitignores:
-            gitignores = UpdateGitignoreAction.find_gitignores(file)
-
-        for gitignore in gitignores:
-            with open(gitignore, "r", encoding="utf-8") as gitignore_file:
-                gitignore_content = gitignore_file.read().splitlines()
-
-            inversed_gitignore_content = []
-            for pattern in gitignore_content:
-                if pattern.startswith("!"):
-                    inversed_gitignore_content.append(pattern[1:])
-
-            relative_file = UpdateGitignoreAction._get_relative_path(file, gitignore)
-
-            zgitignore_helper = zgitignore.ZgitIgnore(gitignore_content)
-            inversed_zgitignore_helper = zgitignore.ZgitIgnore(inversed_gitignore_content)
-            if zgitignore_helper.is_ignored(relative_file) or inversed_zgitignore_helper.is_ignored(relative_file):
-                return True, relative_file, gitignore, gitignore_content
-        return False, None, None, None
-
-    @staticmethod
-    def remove(file: str):
-        """
-        Remove an ignored file
-        :param file:
-        :return:
-        """
-        ignored, relative_file, gitignore, gitignore_content = UpdateGitignoreAction.is_file_ignored(file)
-        if not ignored:
-            return
-
-        new_gitignore_content = list(filter(lambda line: relative_file != line, gitignore_content))
-        before_block = UpdateGitignoreAction.get_block_limit(True)
-        after_block = UpdateGitignoreAction.get_block_limit(False)
-        excluded = [before_block, after_block]
-        if new_gitignore_content and new_gitignore_content != excluded:
-            UpdateGitignoreAction._cleanup_gitignore_content(after_block, before_block, new_gitignore_content)
-
-            with open(gitignore, "w", encoding="utf-8") as gitignore_file:
-                for line in new_gitignore_content:
-                    gitignore_file.write(line)
-                    gitignore_file.write("\n")
-        else:
-            force_remove(gitignore)
-
-        context.log.warning("%s removed from %s", file, gitignore)
-
-    @staticmethod
-    def _cleanup_gitignore_content(after_block, before_block, new_gitignore_content):
-        after_block_row = new_gitignore_content.index(after_block)
-        before_block_row = new_gitignore_content.index(before_block)
-        if after_block_row > -1 and before_block_row > -1 and after_block_row - before_block_row == 1:
-            new_gitignore_content.remove(after_block)
-            new_gitignore_content.remove(before_block)
-        while new_gitignore_content and not new_gitignore_content[0].strip():
-            new_gitignore_content.pop(0)
-
-        while new_gitignore_content and not new_gitignore_content[-1].strip():
-            new_gitignore_content.pop(-1)
-
-    @staticmethod
-    def _get_relative_path(target: str, gitignore: str, first_slash: bool = True):
-        relpath = pathlib.Path(os.path.relpath(target, os.path.dirname(gitignore))).as_posix()
-        if first_slash:
-            return "/" + relpath
-        return relpath
-
     @staticmethod
     def execute(target: str, source: str = None):
         """
         Execute action
         """
-        ignored, _, _, _ = UpdateGitignoreAction.is_file_ignored(target)
+        ignored, _, _, _, _, _ = UpdateGitignoreAction._check_file_ignore(target)
         if ignored:
             return
 
         try:
-            gitignore = next(UpdateGitignoreAction.find_gitignores(target))
+            gitignore = next(UpdateGitignoreAction._find_gitignores(target))
         except StopIteration:
             gitignore = ".gitignore"
 
         relative_target = UpdateGitignoreAction._get_relative_path(target=target, gitignore=gitignore, first_slash=True)
-        UpdateGitignoreAction.add_file(gitignore, relative_target)
+
+        # Retrieve the block lines
+        block_lines, before_lines, after_lines = UpdateGitignoreAction._read_and_split_gitignore_file(gitignore)
+
+        # Add of the relative_target to the block and sort the whole block.
+        block_lines.append(relative_target)
+
+        UpdateGitignoreAction._sanitize_block(block_lines, before_lines, after_lines)
+        UpdateGitignoreAction._write_gitignore_content(gitignore, before_lines + block_lines + after_lines)
         context.log.success("%s added to %s", relative_target, gitignore)
 
     @staticmethod
@@ -161,71 +84,160 @@ class UpdateGitignoreAction(InitializableAction):
         cache.flush()
 
     @staticmethod
-    def add_file(gitignore: str, relative_target: str):
+    def remove(file: str):
         """
-        Add a file to the gitignore block
-        :param gitignore: the file to update
-        :param relative_target: the file to add into the block
+        Remove an ignored file
+        :param file:
         :return:
         """
-        # Retrieval of file content
-        gitignore_file_content = ''
-        if os.path.exists(gitignore):
-            with open(gitignore, "r+", encoding="utf-8") as gitignore_file:
-                gitignore_file_content = gitignore_file.read()
+        ignored, target_file, gitignore, block_lines, before_lines, after_lines = \
+            UpdateGitignoreAction._check_file_ignore(file)
+        if not ignored:
+            return
 
-        # Retrieval of the current block
-        current_block = UpdateGitignoreAction.get_block(gitignore_file_content)
-        is_block_present = (len(current_block) > 0)
+        # Removal of the target_file to the block and sort the whole block.
+        block_lines.remove(target_file)
 
-        # Creation of the new block version
-        if not is_block_present:
-            new_block = [UpdateGitignoreAction.get_block_limit(True), UpdateGitignoreAction.get_block_limit(False)]
-        else:
-            new_block = current_block.copy()
+        UpdateGitignoreAction._sanitize_block(block_lines, before_lines, after_lines)
+        UpdateGitignoreAction._write_gitignore_content(gitignore, before_lines + block_lines + after_lines)
 
-        # Addition of the target_file to the block
-        new_block.insert(len(current_block) - 1, relative_target)
-
-        # Updating the gitignore content
-        if is_block_present:
-            gitignore_file_content = gitignore_file_content.replace('\n'.join(current_block), '\n'.join(new_block))
-        else:
-            if len(gitignore_file_content) > 0:
-                gitignore_file_content += '\n'
-            gitignore_file_content += '\n'.join(new_block) + '\n'
-
-        # Writing new content to the file
-        with open(gitignore, "w+", encoding="utf-8") as gitignore_file:
-            gitignore_file.write(gitignore_file_content)
+        context.log.warning("%s removed from %s", file, gitignore)
 
     @staticmethod
-    def get_block(gitignore_file_content: str):
+    def _get_relative_path(target: str, gitignore: str, first_slash: bool = True):
+        relpath = pathlib.Path(os.path.relpath(target, os.path.dirname(gitignore))).as_posix()
+        if first_slash:
+            return "/" + relpath
+        return relpath
+
+    @staticmethod
+    def _find_gitignores(target: str):
         """
-        Retrieve the gitignore block generated in the file if exist
-        :param gitignore_file_content: the gitignore content
+        Find writable .gitignore files, walking directories up to project home.
+        """
+        dirname = target
+
+        while os.path.abspath(dirname).startswith(config.paths.project_home) and \
+                os.path.abspath(dirname) != os.path.abspath(config.paths.project_home):
+            dirname = os.path.dirname(dirname)
+            gitignore = os.path.join(dirname, ".gitignore")
+            if os.path.exists(gitignore) and os.access(gitignore, os.W_OK):
+                yield gitignore
+
+    @staticmethod
+    def _check_file_ignore(file: str, gitignores=None):
+        """
+        Check if a file is ignore, going throw all provided gitignore files
+        :param file:
+        :param gitignores:
         :return:
         """
-        started = False
-        ended = False
-        lines = []
-        for line in gitignore_file_content.split('\n'):
-            if line == UpdateGitignoreAction.get_block_limit(True):
-                started = True
-            if started and not ended:
-                lines.append(line)
-            if line == UpdateGitignoreAction.get_block_limit(False):
-                ended = True
+        if not gitignores:
+            gitignores = UpdateGitignoreAction._find_gitignores(file)
 
-        return lines
+        for gitignore in gitignores:
+            block_lines, before_lines, after_lines = UpdateGitignoreAction._read_and_split_gitignore_file(gitignore)
+
+            gitignore_content = before_lines + block_lines + after_lines
+
+            inversed_gitignore_content = []
+            for pattern in gitignore_content:
+                if pattern.startswith("!"):
+                    inversed_gitignore_content.append(pattern[1:])
+
+            relative_file = UpdateGitignoreAction._get_relative_path(file, gitignore)
+
+            zgitignore_helper = zgitignore.ZgitIgnore(gitignore_content)
+            inversed_zgitignore_helper = zgitignore.ZgitIgnore(inversed_gitignore_content)
+            if zgitignore_helper.is_ignored(relative_file) or inversed_zgitignore_helper.is_ignored(relative_file):
+                return True, relative_file, gitignore, block_lines, before_lines, after_lines
+        return False, None, None, None, None, None
 
     @staticmethod
-    def get_block_limit(start: bool):
+    def _get_block_limit(start: bool):
         """
         Generate the gitignore block limit
         :param start: if it is the start of the block or not
         :return:
         """
+        markers = config.data.get('gitignore.markers', ['inetum-orleans/docker-devbox'])
+
         if start:
-            return '###> {} ###'.format(repository)
-        return '###< {} ###'.format(repository)
+            return f'###> {markers[0]} ###'
+        return f'###< {markers[0]} ###'
+
+    @staticmethod
+    def _is_block_limit(line: str, start: Optional[bool] = None):
+        """
+        Check if line match a block limit.
+        :param line: line to check
+        :param start: if it is the start of the block or not
+        :return:
+        """
+        markers = config.data.get('gitignore.markers', ['inetum-orleans/docker-devbox'])
+
+        for marker in markers:
+            if start or start is None:
+                if f'###> {marker} ###' == line:
+                    return True
+            if not start or start is None:
+                if f'###< {marker} ###' == line:
+                    return True
+        return False
+
+    @staticmethod
+    def _write_gitignore_content(gitignore: str, content: List[str]):
+        if [line for line in content if line]:
+            with open(gitignore, "w", encoding="utf-8") as gitignore_file:
+                for lineno, line in enumerate(content):
+                    if lineno > 0:
+                        gitignore_file.write('\n')
+                    gitignore_file.write(line)
+        else:
+            force_remove(gitignore)
+
+    @staticmethod
+    def _split_block_lines(gitignore_file_content: str):
+        started = False
+        ended = False
+        before_lines = []
+        content_lines = []
+        after_lines = []
+        for line in gitignore_file_content.split('\n'):
+            if UpdateGitignoreAction._is_block_limit(line, False):
+                ended = True
+            if not started and not ended:
+                before_lines.append(line)
+            elif started and not ended:
+                content_lines.append(line)
+            elif started and ended:
+                after_lines.append(line)
+            if UpdateGitignoreAction._is_block_limit(line, True):
+                started = True
+
+        return content_lines, before_lines, after_lines
+
+    @staticmethod
+    def _read_and_split_gitignore_file(gitignore_file: str):
+        if os.path.exists(gitignore_file):
+            with open(gitignore_file, "r+", encoding="utf-8") as gitignore_fp:
+                return UpdateGitignoreAction._split_block_lines(gitignore_fp.read())
+        else:
+            return [], [], []
+
+    @staticmethod
+    def _sanitize_block(block_lines, before_lines, after_lines):
+        block_lines = [line for line in block_lines if line]
+        block_lines.sort()
+
+        if block_lines:
+            if not before_lines or not UpdateGitignoreAction._is_block_limit(before_lines[-1], start=True):
+                before_lines.append(UpdateGitignoreAction._get_block_limit(start=True))
+
+            if not after_lines or not UpdateGitignoreAction._is_block_limit(after_lines[0], start=False):
+                after_lines.insert(0, UpdateGitignoreAction._get_block_limit(start=False))
+        else:
+            if before_lines and UpdateGitignoreAction._is_block_limit(before_lines[-1], start=True):
+                before_lines.pop(-1)
+            if after_lines and UpdateGitignoreAction._is_block_limit(after_lines[0], start=False):
+                after_lines.pop(0)
