@@ -291,27 +291,6 @@ class TestDockerFeature:
         assert os.path.exists(os.path.join(config.paths.home, "certs", "web-changed.domain.tld.crt"))
         assert os.path.exists(os.path.join(config.paths.home, "traefik", "config", "web-changed.domain.tld.ssl.toml"))
 
-    @pytest.mark.skipif("os.name == 'nt'")
-    def test_named_user_group(self, project_loader):
-        project_loader("empty")
-
-        features.register(DockerFeature())
-        load_registered_features()
-
-        assert config.data.get('docker.user.name_to_uid')
-        assert config.data.get('docker.user.group_to_gid')
-
-    @pytest.mark.skipif("os.name != 'nt'")
-    def test_named_user_group_windows(self, project_loader):
-        project_loader("empty")
-
-        features.register(DockerFeature())
-        load_registered_features()
-
-        assert config.data.get('docker.user.name_to_uid') == {}
-        assert config.data.get('docker.user.group_to_gid') == {}
-
-
 class TestDockerDisplayInfoAction:
 
     def test_retrieve_environment_data(self):
