@@ -6,7 +6,8 @@ import pytest
 import yaml
 
 from ddb.__main__ import load_registered_features, register_actions_in_event_bus
-from ddb.config import config
+from ddb.config import config, migrations
+from ddb.config.migrations import PropertyMigration
 from ddb.feature import features
 from ddb.feature.core import CoreFeature
 from ddb.feature.docker import DockerFeature
@@ -40,6 +41,26 @@ class TestJsonnetAction:
         action.initialize()
         action.execute()
 
+    @pytest.mark.skipif("os.name == 'nt'")
+    def test_named_user_group(self, project_loader):
+       project_loader("empty")
+
+       features.register(JsonnetFeature())
+       load_registered_features()
+
+       assert config.data.get('jsonnet.docker.user.name_to_uid')
+       assert config.data.get('jsonnet.docker.user.group_to_gid')
+
+    @pytest.mark.skipif("os.name != 'nt'")
+    def test_named_user_group_windows(self, project_loader):
+       project_loader("empty")
+
+       features.register(JsonnetFeature())
+       load_registered_features()
+
+       assert config.data.get('jsonnet.docker.user.name_to_uid') == {}
+       assert config.data.get('jsonnet.docker.user.group_to_gid') == {}
+
     def test_example1(self, project_loader):
         project_loader("example1")
 
@@ -390,8 +411,8 @@ class TestJsonnetAction:
 
         assert rendered == expected
 
-    def test_docker_compose_disabled_services(self, project_loader):
-        project_loader("docker_compose_disabled_services")
+    def test_docker_compose_excluded_services(self, project_loader):
+        project_loader("docker_compose_excluded_services")
 
         features.register(CoreFeature())
         features.register(FileFeature())
@@ -414,6 +435,30 @@ class TestJsonnetAction:
 
         assert rendered == expected
 
+    def test_docker_compose_included_services(self, project_loader):
+            project_loader("docker_compose_included_services")
+
+            features.register(CoreFeature())
+            features.register(FileFeature())
+            features.register(DockerFeature())
+            features.register(JsonnetFeature())
+            load_registered_features()
+            register_actions_in_event_bus(True)
+
+            action = FileWalkAction()
+            action.initialize()
+            action.execute()
+
+            assert os.path.exists('docker-compose.yml')
+            with open('docker-compose.yml', 'r') as f:
+                rendered = yaml.load(f.read(), yaml.SafeLoader)
+
+            with open('docker-compose.expected.yml', 'r') as f:
+                expected_data = f.read()
+                expected = yaml.load(expected_data, yaml.SafeLoader)
+
+            assert rendered == expected
+
     @pytest.mark.parametrize("variant", [
         "_register_binary",
         "_register_binary_with_one_option",
@@ -494,4 +539,51 @@ class TestJsonnetAction:
 
             expected = yaml.load(expected_data, yaml.SafeLoader)
 
-        assert rendered == expected
\ No newline at end of file
+        assert rendered == expected
+
+
+class TestJsonnetAutofix:
+    def teardown_method(self, test_method):
+        migrations.set_history()
+
+    def test_autofix_variables_only(self, project_loader):
+        project_loader("autofix_variables_only")
+
+        config.args.autofix = True
+
+        history = (
+            PropertyMigration("old_property",
+                              "new_property", since="v1.1.0"),
+            PropertyMigration("some.deep.old.property",
+                              "some.another.new.property", since="v1.1.0"),
+        )
+
+        migrations.set_history(history)
+
+        features.register(CoreFeature())
+        features.register(FileFeature())
+        features.register(DockerFeature())
+        features.register(JsonnetFeature())
+        load_registered_features()
+        register_actions_in_event_bus(True)
+
+        action = FileWalkAction()
+        action.initialize()
+        action.execute()
+
+        assert os.path.exists('variables.json')
+        with open('variables.json', 'r') as f:
+            rendered = f.read()
+
+        with open('variables.expected.json', 'r') as f:
+            expected = f.read()
+
+        assert expected == rendered
+
+        with open('variables.json.jsonnet', 'r') as f:
+            source = f.read()
+
+        with open('variables.json.autofix', 'r') as f:
+            fixed = f.read()
+
+        assert source == fixed
